#ifndef LPC_MEMPOOL_H1103129409_INCLUDE_GUARD_
#define LPC_MEMPOOL_H1103129409_INCLUDE_GUARD_

/* This file is part of indexlib.
 * Copyright (C) 2005 Lus Pedro Coelho <luis@luispedro.org>
 *
 * Indexlib is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation and available as file
 * GPL_V2 which is distributed along with indexlib.
 * 
 * Indexlib is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * In addition, as a special exception, the copyright holders give
 * permission to link the code of this program with any edition of
 * the Qt library by Trolltech AS, Norway (or with modified versions
 * of Qt that use the same license as Qt), and distribute linked
 * combinations including the two.  You must obey the GNU General
 * Public License in all respects for all of the code used other than
 * Qt.  If you modify this file, you may extend this exception to
 * your version of the file, but you are not obligated to do so.  If
 * you do not wish to do so, delete this exception statement from
 * your version.
 */


#include "memreference.h"
#include "manager.h"
#include "memreference.h"
#include <cassert>
#include <vector>
#include <memory>
#include <algorithm>
#include <ostream>

template <typename Traits>
struct mempool /* : boost::noncopyable */ {
	public:
		typedef Traits traits_type;
		typedef typename traits_type::value_type data_type;
		typedef typename traits_type::pointer data_typeptr;
		explicit mempool( std::auto_ptr<memory_manager> source );
		data_typeptr allocate( unsigned );
		data_typeptr reallocate( data_typeptr, unsigned );
		void deallocate( data_typeptr );

		void print( std::ostream& out ) const;

		unsigned size() const { return manager_->size(); }
	private:
		static unsigned order_of( unsigned x ) {
			assert( x > 0 );
			if ( x < traits_type::min_size() ) x = traits_type::min_size();
			--x;
			unsigned res = 0;
			while ( x ) {
				++res;
				x >>= 1;
			}
			return res;
		}
		static unsigned order_to_size( unsigned order ) {
			return 1 << order;
		}
		static unsigned size_of( data_typeptr data ) {
			return traits_type::size_of( data );
		}

		struct list_node {
			uint32_t order;
			uint32_t next;
			uint32_t prev;
		};
		enum { min_order_for_free_node = 4 };
		
		list_node* get_node( uint32_t p ) const;
		memory_reference<uint32_t> free_list( unsigned order );
		uint32_t free_list( unsigned order ) const {
			return const_cast<mempool*>( this )->free_list( order );
		}
		void insert_into_list( uint32_t where, unsigned order );
		void remove_from_list( uint32_t where, unsigned order );
		void break_up( uint32_t where );
		void init_memory();
		void fill_into_list( unsigned old_size );
		void fill_into_list( unsigned old_size, unsigned order );

		bool join( data_typeptr&, unsigned order );
		void deallocate( data_typeptr, unsigned order );

		std::auto_ptr<memory_manager> manager_;
		unsigned max_order_;
};

#include "mempool.tcc"

#endif /* LPC_MEMPOOL_H1103129409_INCLUDE_GUARD_ */
