#ifndef LPC_TREE_H1103129409_INCLUDE_GUARD_
#define LPC_TREE_H1103129409_INCLUDE_GUARD_

/* This file is part of indexlib.
 * Copyright (C) 2005 Lus Pedro Coelho <luis@luispedro.org>
 *
 * Indexlib is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation and available as file
 * GPL_V2 which is distributed along with indexlib.
 * 
 * Indexlib is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * In addition, as a special exception, the copyright holders give
 * permission to link the code of this program with any edition of
 * the Qt library by Trolltech AS, Norway (or with modified versions
 * of Qt that use the same license as Qt), and distribute linked
 * combinations including the two.  You must obey the GNU General
 * Public License in all respects for all of the code used other than
 * Qt.  If you modify this file, you may extend this exception to
 * your version of the file, but you are not obligated to do so.  If
 * you do not wish to do so, delete this exception statement from
 * your version.
 */


#include "reference.h"
#include "node.h"
#include "character.h"
#include "leafdata.h"
#include "mempool.h"
#include "logfile.h"
#include "stringarray.h"
#include "index.h"
#include <boost/format.hpp>
#include <boost/tuple/tuple.hpp>
#include <vector>

using node::nodeptr;
struct tree_result : indexlib::result {
	public:
		explicit tree_result( nodeptr n = nodeptr() );
		std::vector<unsigned> list() const;
		std::auto_ptr<indexlib::result> search( const char* ) { return std::auto_ptr<result>(); }
		unsigned size() const;
		bool empty() const;
	private:
		void dfs_fill( nodeptr p );
		std::vector<unsigned> results_;
};

struct tree : indexlib::index {
	public:
		tree( std::string base );
		~tree();
		void add( const std::string, std::string docname );
		std::auto_ptr<indexlib::result> search( std::string ) const;
		std::string lookup_docname( unsigned idx ) const {
			logfile() << boost::format( "tree::lookup_docname( %s ): %s\n" ) % idx % docnames_.get( idx );
			return docnames_.get( idx );
		}
		unsigned ndocs() const { return docnames_.size(); }
	private:
		boost::tuple<nodeptr, const character*> dive( nodeptr, const character* ) const;
		boost::tuple<nodeptr, const character*> dive( const character* str ) const { return dive( root_, str ); }
		template<typename Node>
			pointer<Node> allocate();
		void deallocate( nodeptr p ) { nodepool_.deallocate( p ); }


		struct inserter;
		friend class inserter;


		node::internalnode_fullptr root_;
		mempool<node::node_mem_traits> nodepool_;
		mempool<leaf_data_pool_traits> leafdatapool_;

		stringarray docnames_;
		stringarray rawstrings_;
		stringarray translatedstrings_;
		std::string filebase_;
};


#endif /* LPC_TREE_H1103129409_INCLUDE_GUARD_ */
